# coding: utf-8
import os, sys, traceback
import wx
import wx.lib.delayedresult as delayedresult
import socket

class Appl(wx.Frame):
	def __init__(self):
		wx.Frame.__init__(self, None, id = wx.ID_ANY, title = u'端口扫描工具', size = (516, 364), style=wx.CAPTION | wx.CLOSE_BOX | wx.MINIMIZE_BOX)

		# self.SetIcon(wx.Icon('appl.exe', wx.BITMAP_TYPE_ICO))
		# self.SetIcon(wx.Icon(appIcon, wx.BITMAP_TYPE_ANY))

		self.panel = wx.Panel(self)

		self.abortEvent = delayedresult.AbortEvent()

		self.hostTextCtrl = wx.TextCtrl(self.panel, value = '127.0.0.1', size = (200, -1))
		self.portSpinCtrl = wx.SpinCtrl(self.panel, value = '80', min=1, max=65535, size = (50, -1))
		self.scanTimeoutSpinCtrl = wx.SpinCtrl(self.panel, value = '1', min=1, max=65535, size = (50, -1))
		self.scanButton = wx.Button(self.panel, label = u'扫描', size = (-1, 25))
		self.aboutButton = wx.Button(self.panel, label = u'关于', size = (-1, 25))
		self.resultsTextCtrl = wx.TextCtrl(self.panel, style = wx.TE_MULTILINE|wx.TE_RICH2 | wx.TE_READONLY)

		# sizer = wx.StaticBoxSizer(wx.StaticBox(self.panel), orient=wx.VERTICAL)
		sizer = wx.BoxSizer(wx.VERTICAL)
		innerSizer = wx.GridBagSizer(0, 8)
		innerSizer.SetEmptyCellSize((0, 0))
		innerSizer.Add(wx.StaticText(self.panel, label = 'Host:'), pos = (1, 0))
		innerSizer.Add(self.hostTextCtrl, pos = (2, 0), flag = wx.ALL | wx.EXPAND)
		innerSizer.Add(wx.StaticText(self.panel, label = 'Port:'), pos = (1, 1))
		innerSizer.Add(self.portSpinCtrl, pos = (2, 1), flag = wx.ALL | wx.EXPAND)
		innerSizer.Add(wx.StaticText(self.panel, label = 'Timeout:'), pos = (1, 2))
		innerSizer.Add(self.scanTimeoutSpinCtrl, pos = (2, 2), flag = wx.ALL | wx.EXPAND)
		innerSizer.Add(self.scanButton, pos = (2, 3))

		innerSizer.Add(self.aboutButton, pos = (2, 4))

		sizer.Add(innerSizer, flag = wx.ALL | wx.EXPAND)
		sizer.Add((-1, 8))
		sizer.Add(self.resultsTextCtrl, 1, wx.EXPAND)

		self.panel.SetSizerAndFit(sizer)
		self.panel.SetClientSize(sizer.GetSize())
		self.Show()
		# 全屏显示
		# self.ShowFullScreen(True, style=wx.FULLSCREEN_ALL)
		self.scanButton.Bind(wx.EVT_BUTTON, self.onScan)
		self.aboutButton.Bind(wx.EVT_BUTTON, self.onAbout)

	def updateResults(self, message):
		self.resultsTextCtrl.AppendText(message + '\n')

	def onScan(self, event):
		self.startSession()
		delayedresult.startWorker(consumer = self._consumer, workerFn = self._worker)

	def onAbout(self, event):
		info = wx.AboutDialogInfo()
		info.Name = u'端口扫描工具'
		info.Version = '1.0'
		info.Copyright = u'版权：xiaozi'
		info.Description = u'wxpython的练手作品，源代码部分来自网络。'
		info.WebSite = ('http://type.so', 'http://type.so')
		# info.Developers = ['%s <%s>' % (u'小子欠扁', '245565986@qq.com')]
		info.Artists = []
		wx.AboutBox(info)

	def _worker(self):
		host = self.hostTextCtrl.GetValue()
		port = self.portSpinCtrl.GetValue()
		timeout = self.scanTimeoutSpinCtrl.GetValue()

		wx.CallAfter(self.updateResults, 'Scaning port %s on host %s, timeout %s.' % (port, host, timeout))

		sk = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
		sk.settimeout(5)
		status = sk.connect_ex((host, port))
		sk.close()

		if status == 0:
			wx.CallAfter(self.updateResults, '  Port %s is open.' % port)

		wx.CallAfter(self.updateResults, 'End scaning.')

	def _consumer(self, result):
		try:
			result.get()
		except Exception:
			traceback.print_exc()
		self.endSession()

	def startSession(self):
		self.resultsTextCtrl.Clear()
		self.scanButton.Enable(False)
		self.abortEvent.clear()

	def endSession(self):
		self.scanButton.Enable(True)


if __name__ == '__main__':
	app = wx.App(redirect = False)
	frame = Appl()
	app.MainLoop()
